#!/usr/bin/python3
# coding=utf-8

###################
# Author: Chunli Wang
# Date: 2024-3-18
# Platform: Kylin Desktop V10 SP1 2303 Release
# Description: 命令行
# Version: 1.0
###################

import argparse
import getpass
import os.path
import shutil
from datetime import datetime

from src.utils.operations import Operation
from src.utils.utils import check_passwd
from src.utils.logger import logger
from src.setting import PASSWORD_RETRY_TIMES


# 转换参数
def parse_args():
    parser = argparse.ArgumentParser(description='This Program is Kylin-Adaptation-Tools',
                                     prog='python3 KAT-cli.py')
    parser.add_argument('-t', '--type', action='store', type=str, required=True, help='Choose Package Type ID: 1-软件, '
                                                                                      '2-外设')
    parser.add_argument('package', action='store', type=str, help='package path')
    return parser.parse_args()


# 收集结果
def collect_report(report_html, des_path):
    shutil.copy2(report_html, des_path)


if __name__ == '__main__':
    # 获取参数
    args = parse_args()

    # 参数校验
    if args.type not in ['1','2']:
        logger.error("请输入正确的软件类型")
        exit()
    else:
        if args.type == '1':
            args.type = 'software'
        else:
            args.type = 'peripheral'

    if not os.path.isfile(args.package):
        logger.error("请输入一个包文件")
        exit()

    # 获取密码
    correct = False
    for i in range(PASSWORD_RETRY_TIMES):
        password = getpass.getpass("请输入密码：")
        # 检测密码
        if not password:
            logger.error("请输入密码")
            continue
        res = check_passwd(password)
        if not res:
            logger.error("密码输入错误，请重试")
        else:
            correct = True
            break

    if not correct:
        exit()

    # 执行测试
    logger.info("测试开始")
    logger.info("测试过程中请勿操作测试环境，避免影响测试结果")
    debcheck_report_detail, debcheck_report_summary, report_pkg_info = Operation().run_debcheck(args.package, args.type)
    report_env, report_case = Operation().run_blazers(args.package, password)
    logger.info("测试完成")

    # 生成报告
    logger.info("生成报告中")
    report_html = Operation().run_report_tool(report_env, debcheck_report_detail, report_case, report_pkg_info)

    des_path = 'Reports'
    des_report = des_path + '/' + report_html.split('/')[-1]

    # 收集报告
    collect_report(report_html, des_path)

    # 提示报告路径
    logger.info(f"报告路径为{des_report}")

    today = datetime.now().strftime("%Y-%m-%d")
    logger.info(f'日志路径为Logs/{today}_info.log和Logs/{today}_error.log')
