#!/bin/bash
#######################################################################################
#Function: setup of applications
#Platform:all
#Author: Fenfen Liu, Chunli Wang
#Date: 2022/3/16
#Company: Kylin Software Co.,Ltd.
#Description: 应用所有用例所需前置条件
########################################################################################
source ./pub/baselib.sh
source ./config/configuration.conf
source ./pub/log.sh

# 功能：解析用例输入参数:包路径。
# 入参（param1：字符串（用例的输入参数包路径）；
# 出参：数组：[0,包存储路径,,softname,packagename version]
function parsing_input_parameter_for_packagepath() {
    local input_parameter=$1
    log_info  "输入参数为包全路径${input_parameter}" >> "${CASE_LOG_FILE}"
    # 获取应用名称、包名、版本号
    local softname=${input_parameter##*/}
    if [ ${SYSTEM} == desktop ]; then
        local packagename=$(dpkg -f "${input_parameter}" "Package")
        local version=$(dpkg -f "${input_parameter}" "Version")
        local release="null"
    else
        local packagename=$(rpm -qi "${input_parameter}" | grep "^Name" | awk -F : '{print $2}' | sed -e 's/ //g')
        local version=$(rpm -qi "${input_parameter}" | grep "^Version" | awk -F : '{print $2}' | sed -e 's/ //g')
        local release=$(rpm -qi "${input_parameter}" | grep "^Release" | awk -F : '{print $2}' | sed -e 's/ //g')
    fi
    # 获取包全路径
    local pkg_path="${PROJECT_PATH}/resource/${packagename}/"
    mkdir ${pkg_path} > /dev/null 2>&1
    cp ${input_parameter} ${pkg_path} > /dev/null 2>&1
    local pkg_full_path="${pkg_path}${softname}"
    log_info "解析参数：包存放路径：${pkg_full_path}, 名称：${softname}, 包名:${packagename},版本号：${version},release版本号：${release}" >> "${CASE_LOG_FILE}"
    echo "0" "${pkg_full_path}" "${softname}" "${packagename}" "${version}" "${release}"
}


# 功能：解析用例输入参数:包名
# 入参（param1：字符串（用例的输入参数包名）；
# 出参：数组：[1,包存储路径,,softname,packagename version]
function parsing_input_parameter_for_packagename() {
    log_info  "输入参数为包名${input_parameter}" >> "${CASE_LOG_FILE}"
    local packagename=${input_parameter}
    # 根据包名下载deb包
    local pkg_path="${PROJECT_PATH}/resource/${packagename}/"
    mkdir ${pkg_path}  > /dev/null 2>&1
    log_info  "下载应用包${input_parameter}到${pkg_path}路径下" >> "${CASE_LOG_FILE}"
    cd ${pkg_path}
    if [ ${SYSTEM} == desktop ]; then
        apt-get download "${packagename}" >> "${CASE_LOG_FILE}"
    else
        yum download "${packagename}" >> "${CASE_LOG_FILE}"
    fi
    if [ $? -eq 0 ];then
        log_info  "下载应用包${input_parameter}成功" >> "${CASE_LOG_FILE}"
        cd - > /dev/null 2>&1
        if [ ${SYSTEM} == desktop ]; then
            local softname=$(ls "${pkg_path}"| grep ".deb$")
            local pkg_full_path="${pkg_path}${softname}"
            local version=$(dpkg -f "${pkg_full_path}" "Version")
            local release="null"
        else
            local softname=$(ls "${pkg_path}"| grep ".rpm$")
            local pkg_full_path="${pkg_path}${softname}"
            local version=$(rpm -qi "${pkg_full_path}" | grep "^Version" | awk -F : '{print $2}' | sed -e 's/ //g')
            local release=$(rpm -qi "${pkg_full_path}" | grep "^Release" | awk -F : '{print $2}' | sed -e 's/ //g')
        fi
        log_info "解析参数：包存放路径：${pkg_full_path}, 名称：${softname}, 包名:${packagename},版本号：${version},release版本号：${release}" >> "${CASE_LOG_FILE}"
    else
        log_error  "下载应用包${input_parameter}失败" >> "${CASE_LOG_FILE}"
    fi
    echo "1" "${pkg_full_path}" "${softname}" "${packagename}" "${version}" "${release}"
}

# 功能：解析用例输入参数:包名还是包路径。
# 入参（param1：字符串（用例的输入参数）；
# 出参：数组：[0|1,包存储路径,,softname,packagename version]，其中0表示入参为包路径，1表示入参为包名
function parsing_input_parameter() {
    local input_parameter=$1
    if [[ $(echo "${input_parameter}" | grep -E '^[1-9][0-9]*$') ]]; then
        log_error "项目输入参数不符合规范，应为包名或包路径，请查看blazers/control/control.ini中的配置项是否正确" >> "${CASE_LOG_FILE}"
    else
        echo ${input_parameter} | grep -E "(deb|rpm)$" > /dev/null
        if [ "$?" = "0" ]; then
            local pkg_info=($(parsing_input_parameter_for_packagepath ${input_parameter}))
        else
            local pkg_info=($(parsing_input_parameter_for_packagename ${input_parameter}))
        fi
        echo ${pkg_info[@]}
    fi
}

# 将初始化后的变量写入配置文件
# 入参（param1：数组（参数类型、deb包存储路径、应用名称、应用包名、应用版本）；
function write_global_variable_to_config()
{
    local pkg_info_array=($*)
    config_path="${PROJECT_PATH}/config/config"
    sed -i "s|INPUT_TYPE=.* # 输入参数类型0：包全路径，1：包名|INPUT_TYPE=${pkg_info_array[0]} # 输入参数类型0：包全路径，1：包名|" "${config_path}"
    sed -i "s|PKG_PATH=.* # 包存储路径|PKG_PATH=${pkg_info_array[1]} # 包存储路径|" "${config_path}"
    sed -i "s|PKG_SOFTNAME=.* # 应用名称|PKG_SOFTNAME=${pkg_info_array[2]} # 应用名称|" "${config_path}"
    sed -i "s|PKG_PACKAGENAME=.* # 应用包名|PKG_PACKAGENAME=${pkg_info_array[3]} # 应用包名|" "${config_path}"
    sed -i "s|PKG_VERSION=.* # 应用版本|PKG_VERSION=${pkg_info_array[4]} # 应用版本|" "${config_path}"
    sed -i "s|PKG_RELEASE=.* # 应用release版本|PKG_RELEASE=${pkg_info_array[5]} # 应用release版本|" "${config_path}"
    source ${config_path}
    log_info "修改配置文件完成" | tee -a "$CASE_LOG_FILE"
}

# 执行初始化
# 入参（param1：应用包名或应用包路径）
# 出参：将全局变量写入config文件
function main(){
    echo "###################################应用 $1 开始测试###################################" >> "${CASE_LOG_FILE}"
    echo "#########################应用 $1 前置处理开始#########################" >> "${CASE_LOG_FILE}"
    local pkg_info_array=($(parsing_input_parameter $1))
    log_info "参数解析后的全局变量数组为： ${pkg_info_array[@]}" >> "${CASE_LOG_FILE}"
    # 将全局变量写入config文件
    if [ "${#pkg_info_array[@]}" != "6" ]; then
        log_error "测试应用参数初始化有误，请查看log" | tee -a "${CASE_LOG_FILE}"
        echo "#########################应用 $1 前置处理失败#########################" >> "${CASE_LOG_FILE}"
        return 1
    else
        log_info "测试应用参数初始化成功" | tee -a "${CASE_LOG_FILE}"
        write_global_variable_to_config ${pkg_info_array[*]}
        echo "#########################应用 $1 前置处理完成#########################" >> "${CASE_LOG_FILE}"
        return 0
    fi
}

main $1