#!/usr/bin/python3
# coding=utf-8

###################
# Author: Pengfei Wang
# Date: 2023-11-21
# Platform: Kylin Desktop V10 SP1
# Description: 报告模块
# Version: 1.0
###################
import os
import csv
import json
import logging
import argparse
from datetime import datetime


class Report:
    def __init__(self, env, standardization_report, compatibility_report, package_info='', running_info=''):
        # html配置项
        self._now = datetime.now()
        self._format_time = self._now.strftime("%Y%m%d%H%M%S")
        self._format_datetime = self._now.strftime("%Y-%m-%d %H:%M:%S")
        self.title = "麒麟适配认证工具-测试报告" + self._format_time
        self.header = "麒麟适配认证工具-测试报告  " + self._format_datetime
        # 输出
        current_folder_path = os.path.dirname(os.path.abspath(__file__))
        self.output_filepath = current_folder_path + "/report/麒麟适配认证工具-测试报告-" + self._format_time + ".html"
        self.output_jsonpath = current_folder_path + "/report/result-" + self._format_time + ".json"
        # html模板文件路径
        self.html_template_filepath = current_folder_path + "/template/template.html"
        # csv文件路径
        self.env_filepath = env  # 环境信息
        self.standardization_report_filepath = standardization_report  # 合规性测试报告
        self.compatibility_report_filepath = compatibility_report  # 兼容性测试报告
        self.package_info_filepath = package_info  # 应用信息
        self.running_info_filepath = running_info  # 运行信息
        # html模板框架
        self.html_template = ""
        # 环境信息
        self.env_info = ""
        # 包信息
        self.pkg_info = ""
        # 运行信息
        self.running_info = ""
        # 详细信息-合规性测试
        self.standardization_detail = ""
        # 详细信息-兼容性测试
        self.compatibility_detail = ""
        # 总结果
        self.result = "通过"
        self.result_color = "green"
        # 公司信息
        self.company = ""
        self.year = ""
        # 配置模板
        self._set_html_template()

    def csv_integrate_to_json(self):

        # 读取应用信息
        pkg_dict = {}
        if self.package_info_filepath:
            try:
                with open(self.package_info_filepath, newline='') as csvfile:
                    reader = csv.DictReader(csvfile)
                    for row in reader:
                        pkg_dict = row
            except:
                logging.error(self.package_info_filepath + "文件内容读取失败，请检查文件输入是否正确")

        # 读取环境信息
        env_dict = {}
        if self.env_filepath:
            try:
                with open(self.env_filepath, newline='') as csvfile:
                    reader = csv.reader(csvfile)
                    for row in reader:
                        env_dict[row[0]] = row[1]
            except:
                logging.error(self.env_filepath + "文件内容读取失败，请检查文件输入是否正确")

        # 读取运行信息
        running_dict = {}
        if self.running_info_filepath:
            # TODO 以后有运行信息csv文件时，在此写解析方法
            pass

        # 读取包规范检查内容
        standardization_dict = []
        if self.standardization_report_filepath:
            try:
                with open(self.standardization_report_filepath, newline='') as csvfile:
                    reader = csv.DictReader(csvfile)
                    for row in reader:
                        standardization_dict.append(row)
            except:
                logging.error(self.standardization_report_filepath + "文件内容读取失败，请检查文件输入是否正确")

        # 读取兼容性测试内容
        compatibility_list = []
        if self.compatibility_report_filepath:
            try:
                with open(self.compatibility_report_filepath, newline='') as csvfile:
                    reader = csv.DictReader(csvfile)
                    for row in reader:
                        compatibility_list.append(row)
            except:
                logging.error(self.compatibility_report_filepath + "文件内容读取失败，请检查文件输入是否正确")
        # 汇总
        json_data = {'env_info': env_dict, 'pkg_info': pkg_dict, 'running_info': running_dict,
                     'standardization_detail': standardization_dict, 'compatibility_detail': compatibility_list}
        json_str = json.dumps(json_data, ensure_ascii=False, indent=4)
        with open(self.output_jsonpath, 'w') as f:
            f.write(json_str)
        return json_data

    # 设置环境信息html格式
    def set_env_info(self, env_info_dict):
        env_info_box = "<h2>环境信息</h2>"
        for k, v in env_info_dict.items():
            env = "<div class='info'><span class='style-title'>" + k + "</span><div class='style-txt'>" + v + "</div></div>"
            env_info_box += env
        self.env_info = env_info_box

    # 设置应用信息html格式
    def set_pkg_info(self, pkg_info_dict):
        pkg_info_box = "<h2>应用信息</h2>"
        for k, v in pkg_info_dict.items():
            pkg = "<div class='info'><span class='style-title'>" + k + "</span><div class='style-txt'>" + v + "</div></div>"
            pkg_info_box += pkg
        self.pkg_info = pkg_info_box

    # 设置运行信息html格式
    def set_running_info(self, running_info_dict):
        running_info_box = "<h2>运行信息</h2>"
        for k, v in running_info_dict.items():
            running = "<div class='info'><span class='style-title'>" + k + "</span><div class='style-txt'>" + v + "</div></div>"
            running_info_box += running
        self.running_info = running_info_box

    # 设置合规性检查报告html格式
    def set_standardization_detail(self, standardization_detail_dict):
        standardization_box = "<h2>包合规性测试</h2>"
        for detail in standardization_detail_dict:
            if detail['未修复时会产生影响'] == "":
                detail['未修复时会产生影响'] = "产生兼容性问题"
            if detail['测试结果'] == '通过':
                detail_div = "<div class='detail' style='margin-top: 20px;'><dt class='detail'>" \
                             "<b style='color: green;'>" + detail['测试结果'] + "</b></dt><dd><div>" \
                             + detail['用例名称'] + "</div></dd></div>"
            elif detail['测试结果'] == '不通过':
                # 修改总结果
                self.result = '失败'
                self.result_color = 'red'
                # 错误内容及修改建议
                error_info_content = ''
                for i in detail['发现错误'].split(' '):
                    error_info_content += '<li>' + i + '</li>'
                err_info = "<ul style='list-style-type: disc; margin: 0; margin-left: 120pt;'>" \
                           "<li><div><span class='messageHeading'>发现错误：</span><ul class='ulMessagesNoOverflow'<li >" \
                           + error_info_content + "</li></ul></div></li><li><span>" \
                                                  "<span class='messageHeading'>未修复时会产生影响：</span>" + \
                           detail['未修复时会产生影响'] + "</span></li><li><span><span class='messageHeading'>" \
                                                          "如何修复：</span>" + detail['如何修复'] + "</span></li></ul>"
                detail_div = "<div class='detail' style='margin-top: 20px;'><dt class='detail'>" \
                             "<b style='color: red;'>" + detail['测试结果'] + "</b></dt><dd><div>" \
                             + detail['用例名称'] + "</div></dd>" + err_info + "</div>"
            elif detail['测试结果'] == '警告':
                # 错误内容及修改建议
                error_info_content = ''
                for i in detail['发现错误'].split(' '):
                    error_info_content += '<li>' + i + '</li>'
                err_info = "<ul style='list-style-type: disc; margin: 0; margin-left: 120pt;'>" \
                           "<li><div><span class='messageHeading'>发现错误：</span><ul class='ulMessagesNoOverflow'<li >" \
                           + error_info_content + "</li></ul></div></li><li><span>" \
                                                  "<span class='messageHeading'>未修复时会产生影响：</span>" + \
                           detail['未修复时会产生影响'] + "</span></li><li><span><span class='messageHeading'>" \
                                                          "如何修复：</span>" + detail['如何修复'] + "</span></li></ul>"
                detail_div = "<div class='detail' style='margin-top: 20px;'><dt class='detail'>" \
                             "<b style='color: orange;'>" + detail['测试结果'] + "</b></dt><dd><div>" \
                             + detail['用例名称'] + "</div></dd>" + err_info + "</div>"
            elif detail['测试结果'] == '不涉及':
                detail_div = "<div class='detail' style='margin-top: 20px;'><dt class='detail'>" \
                             "<b style='color: grey;'>" + detail['测试结果'] + "</b></dt><dd><div>" \
                             + detail['用例名称'] + "</div></dd></div>"
            else:
                detail_div = ""
            standardization_box += detail_div
        self.standardization_detail = standardization_box

    # 设置兼容性检查报告html格式
    def set_compatibility_detail(self, compatibility_detail_dict):
        compatibility_box = "<h2>包兼容性测试</h2>"
        for detail in compatibility_detail_dict:
            if detail['未修复时会产生影响'] == "":
                detail['未修复时会产生影响'] = "产生兼容性问题"
            if detail['测试结果'] == '通过':
                detail_div = "<div class='detail' style='margin-top: 20px;'><dt class='detail'>" \
                             "<b style='color: green;'>" + detail['测试结果'] + "</b></dt><dd><div>" \
                             + detail['用例名称'] + "</div></dd></div>"
            elif detail['测试结果'] == '不通过':
                # 修改总结果
                self.result = '失败'
                self.result_color = 'red'
                # 错误内容及修改建议
                error_info_content = ''
                for i in detail['发现错误'].split('\n'):
                    error_info_content += '<li>' + i + '</li>'
                err_info = "<ul style='list-style-type: disc; margin: 0; margin-left: 120pt;'>" \
                           "<li><div><span class='messageHeading'>发现错误：</span><ul class='ulMessagesNoOverflow'<li >" \
                           + error_info_content + "</li></ul></div></li><li><span>" \
                                                  "<span class='messageHeading'>未修复时会产生影响：</span>" + \
                           detail['未修复时会产生影响'] + "</span></li><li><span><span class='messageHeading'>" \
                                                          "如何修复：</span>" + detail['如何修复'] + "</span></li></ul>"
                detail_div = "<div class='detail' style='margin-top: 20px;'><dt class='detail'>" \
                             "<b style='color: red;'>" + detail['测试结果'] + "</b></dt><dd><div>" \
                             + detail['用例名称'] + "</div></dd>" + err_info + "</div>"
            elif detail['测试结果'] == '警告':
                # 错误内容及修改建议
                error_info_content = ''
                for i in detail['发现错误'].split('\n'):
                    error_info_content += '<li>' + i + '</li>'
                err_info = "<ul style='list-style-type: disc; margin: 0; margin-left: 120pt;'>" \
                           "<li><div><span class='messageHeading'>发现错误：</span><ul class='ulMessagesNoOverflow'<li >" \
                           + error_info_content + "</li></ul></div></li><li><span>" \
                                                  "<span class='messageHeading'>未修复时会产生影响：</span>" + \
                           detail['未修复时会产生影响'] + "</span></li><li><span><span class='messageHeading'>" \
                                                          "如何修复：</span>" + detail['如何修复'] + "</span></li></ul>"
                detail_div = "<div class='detail' style='margin-top: 20px;'><dt class='detail'>" \
                             "<b style='color: orange;'>" + detail['测试结果'] + "</b></dt><dd><div>" \
                             + detail['用例名称'] + "</div></dd>" + err_info + "</div>"
            elif detail['测试结果'] == '不涉及':
                detail_div = "<div class='detail' style='margin-top: 20px;'><dt class='detail'>" \
                             "<b style='color: grey;'>" + detail['测试结果'] + "</b></dt><dd><div>" \
                             + detail['用例名称'] + "</div></dd></div>"
            else:
                detail_div = ""
            compatibility_box += detail_div
        self.compatibility_detail = compatibility_box

    def _set_html_template(self):
        with open(self.html_template_filepath, 'r') as f:
            self.html_template = f.read()


    def _set_html_content(self, title, header, env_info, pkg_info, running_info, result,
                          result_color, standardization_detail, compatibility_detail, company, year):
        # 写入模板
        html = self.html_template.format(title=title,
                                         header=header,
                                         env_info=env_info,
                                         pkg_info=pkg_info,
                                         running_info=running_info,
                                         result_color=result_color,
                                         result=result,
                                         standardization_detail=standardization_detail,
                                         compatibility_detail=compatibility_detail,
                                         company=company,
                                         year=year)
        return html

    def output_html(self):
        # 获取全部信息
        data = self.csv_integrate_to_json()
        # 获取环境信息
        self.set_env_info(data['env_info'])
        # 获取应用信息
        if self.package_info_filepath != '':
            self.set_pkg_info(data['pkg_info'])
        # 获取运行信息
        if self.running_info_filepath != '':
            self.set_running_info(data['running_info'])
        # 获取合规性详细内容
        try:
            self.set_standardization_detail(data['standardization_detail'])
        except:
            logging.error(self.standardization_report_filepath + "文件中的字段信息错误，请检查")
        # 获取兼容性详细内容
        try:
            self.set_compatibility_detail(data['compatibility_detail'])
        except:
            logging.error(self.compatibility_report_filepath + "文件中的字段信息错误，请检查")
        # 写入全部内容
        html = self._set_html_content(self.title,
                                      self.header,
                                      self.env_info,
                                      self.pkg_info,
                                      self.running_info,
                                      self.result,
                                      self.result_color,
                                      self.standardization_detail,
                                      self.compatibility_detail,
                                      self.company,
                                      self.year)
        # 写入文件
        with open(self.output_filepath, 'w') as file:
            file.write(html)
        print("HTML报告输出至：" + self.output_filepath)
        print("JSON文件输出至：" + self.output_jsonpath)


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="报告整合及报告生成模块")
    parser.add_argument("-e", "--env_filepath", type=str, help="环境信息csv文件路径(必填)", required=True)
    parser.add_argument("-s", "--standardization_filepath", type=str, help="合规性检查报告csv文件路径(必填)", required=True)
    parser.add_argument("-c", "--compatibility_filepath", type=str, help="兼容性检查报告csv文件路径(必填)", required=True)
    parser.add_argument("-p", "--package_filepath", type=str, help="应用信息csv文件路径(选填)")
    parser.add_argument("-r", "--running_filepath", type=str, help="运行信息csv文件路径(选填)")
    args = parser.parse_args()
    report = Report(args.env_filepath,
                    args.standardization_filepath,
                    args.compatibility_filepath,
                    args.package_filepath,
                    args.running_filepath)
    report.output_html()

