# -*- coding: utf-8 -*-   
""" 
# @Project : kylin-adaptation-tools
# @Author : Code_zhangyikun
# @Time : 2023/8/8 下午5:32
"""
import os
import pprint
import time
from docx import Document
from docx.text.paragraph import Paragraph
from docx.shared import Inches, Cm, RGBColor, Pt
from docx.enum.text import WD_PARAGRAPH_ALIGNMENT
from docx.enum.table import WD_CELL_VERTICAL_ALIGNMENT
from docx.oxml.ns import qn, nsdecls
from docx.oxml import parse_xml


class DocUtils(object):

    def __init__(self, classify):
        """
        :param classify: 生成报告的类型[softwart]/[peripheral]
        """
        self.classify = classify
        # 模板路径字典
        self.tempfile_path = {
            'software': f"{os.path.abspath('Template/software_report_template.docx')}",
            'peripheral': f"{os.path.abspath('Template/peripheral_report_template.docx')}"
        }
        if self.tempfile_path.get(classify) is None:
            raise ValueError("未在tempfile_path中找到classify字段内容")

        self.doc = Document(self.tempfile_path.get(classify))

    @staticmethod
    def _wrap(string, max_width):
        """
        # 将超长字符串按max_width进行截取，添加换行符
        :param string: 待规整的字符串
        :param max_width: 最大长度
        :return: 以最大长度为间隔添加换行符的字符串
        """
        result1 = [string[i:i + max_width] for i in range(0, len(string), max_width)]
        result = '\n'.join(result1)
        return result

    @staticmethod
    def _replace_placeholder_text(doc, params):
        """
        # 替换word中的所有文本占位符
        :param doc: python-docx中的Document对象
        :param params: 占位符字典{'占位符1':'替换的内容1', ...}
        :return: None
        """
        # 遍历文档中的所有段落
        for paragraph in doc.paragraphs:
            for param in params:  # 遍历需要替换的占位符字典
                pv = str(params[param])
                if param in paragraph.text:  # 判断文档段落是否包含该占位符
                    for run in paragraph.runs:  # 保留文档原格式的处理
                        if param in run.text:  # 判断文档段落原格式是否包含该占位符
                            run.text = run.text.replace(param, pv)  # 文本替换
                            run.italic = False
        # 遍历文档中的所有表格
        for table in doc.tables:
            for row in table.rows:  # 遍历表格中的所有行
                for cell in row.cells:  # 遍历行中的所有单元格
                    for param in params:  # 遍历需要替换的占位符字典
                        pv = str(params[param])
                        if param in cell.text:  # 判断单元格是否包含该占位符
                            for paragraph in cell.paragraphs:  # 保留文档原格式的处理
                                for run in paragraph.runs:  # 保留文档原格式的处理
                                    if param in run.text:  # 判断单元格原格式是否包含该占位符
                                        run.text = run.text.replace(param, pv)  # 文本替换
                                        run.italic = False

    @staticmethod
    def _move_table_after(table, paragraph):
        """
        # 将表格移动到指定位置后
        :param table: 目标表格
        :param paragraph: 目标位置
        :return:
        """
        tbl, p = table._tbl, paragraph._p
        p.addnext(tbl)

    @staticmethod
    def _move_table_before(table, paragraph):
        """
        # 将表格移动到指定位置前
        :param table: 目标表格
        :param paragraph: 目标位置
        :return:
            """
        tbl, p = table._tbl, paragraph._p
        p.addprevious(tbl)

    @staticmethod
    def _get_target(document, expect_text):
        """
        # 获取期望段落位置，移动表格前需先调用此方法
        :param document: 目标文档
        :param expect_text:定位的文本内容
        :return: 期望内容在本文档的位置。可搭配move_table_after和move_table_bdfor使用
        """
        for num, paragraph in enumerate(document.paragraphs):
            paragraph_text = paragraph.text
            if paragraph_text.endswith(expect_text):
                target = paragraph
                return target
        return None

    @staticmethod
    def _get_target_next(document, expect_text):
        """
        # 得到目标位置的下一位置
        # 存在的问题，当寻找的段落为最后一个段落是，无法得到此段落的下一位置，返回None
        :param document: 目标文档
        :param expect_text:定位的文本内容
        :return: 期望内容在本文档的下一位置。可搭配move_table_after和move_table_bdfor使用
        """
        i = 0
        for num, paragraph in enumerate(document.paragraphs):
            paragraph_text = paragraph.text
            if i == 1:
                return paragraph
            if paragraph_text.endswith(expect_text):
                i = 1
        return None

    def _create_env_table(self, env):
        """
        # 创建系统基本信息（自动化采集）表格
        :param env: 环境信息dict
        """
        # 定位docutils_basic_system_infor 系统基本信息（自动化采集）位置
        env_target = self._get_target(self.doc, 'docutils_basic_system_infor')
        if env_target is None:
            raise ValueError(
                "未找到 docutils_basic_system_infor 位置，请确认模板文档是否被修改，或联系技术人员进行问题定位")
        env_table = self.doc.add_table(rows=len(env), cols=2, style='Table Grid')
        env_table.style.font.size = Pt(10.5)  # 字体大小10.5磅 对应5号
        n = 0
        for k, v in env.items():
            # 设置单元格宽度
            env_table.cell(n, 0).width = Inches(4)
            env_table.cell(n, 1).width = Inches(6)
            # 向单元格内填充内容
            cell_0 = env_table.cell(n, 0)
            paragraph_0 = cell_0.paragraphs[0]
            run_0 = paragraph_0.add_run(str(k))
            cell_1 = env_table.cell(n, 1)
            paragraph_1 = cell_1.paragraphs[0]
            run_1 = paragraph_1.add_run(str(self._wrap(v, 50)))

            # 设置单元格样式
            cell_0.vertical_alignment = WD_CELL_VERTICAL_ALIGNMENT.CENTER  # 垂直居中
            cell_1.vertical_alignment = WD_CELL_VERTICAL_ALIGNMENT.CENTER  # 垂直居中
            n += 1

        self._move_table_after(env_table, env_target)

    def _create_result_list(self,result):
        """
        # 创建  自动化用例列表及结果  表格
        :param result: 测试结果list
        """
        # 定位docutils_auto_case_result_list 自动化用例列表及结果
        case_target = self._get_target(self.doc, 'docutils_auto_case_result_list')
        case_title = ['测试类别', '用例编号', '用例名称', '测试结果']
        table = self.doc.add_table(rows=1, cols=4, style='Table Grid')
        # table.style.font.size = Pt(9)  # 字体大小10.5磅 对应5号
        # 写表格第一行
        rows = table.rows[0]
        i = 0
        for cell in rows.cells:
            # 第一行背景色设置为蓝色
            shading_elm = parse_xml(r'<w:shd {} w:fill="0080CB"/>'.format(nsdecls('w')))
            cell._tc.get_or_add_tcPr().append(shading_elm)
            paragraph = cell.paragraphs[0]
            run = paragraph.add_run(str(case_title[i]))
            # 字体颜色设置为白色，加粗
            run.font.size = Pt(9)
            run.font.color.rgb = RGBColor.from_string("FFFFFF")  # 字体颜色
            run.font.bold = True  # 加粗
            cell.paragraphs[0].alignment = WD_PARAGRAPH_ALIGNMENT.CENTER  # 水平居中
            cell.vertical_alignment = WD_CELL_VERTICAL_ALIGNMENT.CENTER  # 垂直居中
            i += 1
        # 设置单元格宽度
        table.cell(0, 0).width = Inches(2)
        table.cell(0, 1).width = Inches(1)
        table.cell(0, 2).width = Inches(5)
        table.cell(0, 3).width = Inches(2)
        table.rows[0].height = Cm(0.8)
        # 写入表格内容
        for r_index in range(len(result)):
            table.add_row()
            row_data = result[r_index]
            for c_index in range(1, len(case_title)):
                cell = table.cell(r_index + 1, c_index)
                paragraph = cell.paragraphs[0]
                run = paragraph.add_run(str(row_data[case_title[c_index]]))
                # 设置单元格样式
                cell.paragraphs[0].alignment = WD_PARAGRAPH_ALIGNMENT.CENTER  # 水平居中
                cell.vertical_alignment = WD_CELL_VERTICAL_ALIGNMENT.CENTER  # 垂直居中
            table.rows[r_index + 1].height = Cm(0.8)
        # 合并第一列单元格
        a = table.cell(1, 0)
        b = table.cell(len(result), 0)
        a.merge(b)
        # 填充第一列内容为自动化测试,并设置水平居中,垂直居中
        cell = table.cell(1, 0)
        paragraph = cell.paragraphs[0]
        run = paragraph.add_run('自动化测试')
        cell.paragraphs[0].alignment = WD_PARAGRAPH_ALIGNMENT.CENTER  # 水平居中
        cell.vertical_alignment = WD_CELL_VERTICAL_ALIGNMENT.CENTER  # 垂直居中
        # 移动表格到指定位置
        self._move_table_after(table, case_target)

    def output_document(self, env, result, path=os.path.abspath('.')):
        """
        # 生成自动化测试报告
        :param env: 环境信息dict
        :param result: 测试结果list
        :param path: 生成文档的文件夹
        :return: 生成文档的路径
        """
        output_doc_path = os.path.join(path, f'{self.classify}_adaptation_test_report_{time.strftime("%Y-%m-%d-%H-%M-%S", time.localtime())}.docx')
        # 字体设置（全局）
        '''只更改font.name是不够的，还需要调用._element.rPr.rFonts的set()方法。'''
        self.doc.styles['Normal'].font.name = u'宋体'
        self.doc.styles['Normal']._element.rPr.rFonts.set(qn('w:eastAsia'), u'宋体')
        # 插入env表格
        self._create_env_table(env)
        # 插入自动化用例列表及结果
        self._create_result_list(result)
        # 替换文档内容
        params_text = {
            'docutils_basic_system_infor': '系统基本信息（自动化采集）',
            'docutils_auto_case_result_list': '自动化用例列表及结果'
        }
        self._replace_placeholder_text(self.doc, params_text)

        # 保存文件
        # print(output_doc_path)
        self.doc.save(output_doc_path)

        # 返回保存的路径
        return output_doc_path

# # 外设
# classify = 'peripheral'
# 软件
# classify = 'software'
# env = {
#     "IP": "10.41.124.86",
#     "镜像名称": "Kylin-Desktop-V10-SP1-General-Release-2303-x86_64-2023-04-27 15:15:13",
#     "操作系统版本": "Kylin Linux Desktop V10 (SP1) 20230427",
#     "操作系统架构": "x86_64",
#     "系统内核版本": "5.4.18-85-generic",
#     "CPU型号": "Intel(R) Core(TM) i7-10700 CPU @ 2.90GHz",
#     "CPU核数": "16",
#     "内存容量": "16384 MB",
#     "硬盘容量": "/dev/nvme0n1：238.49 /dev/sda：931.53 GB",
#     "系统分辨率": "1920x1080",
#     "当前显示器": "VGA-0",
#     "显卡型号": "01:00.0 VGA compatible controller: NVIDIA Corporation GK208B [GeForce GT 730] (rev a1)"
# }
#
# result = [{'测试套件': 'Desktop',
#            '测试用时': '00:00:00',
#            '测试结果': 'PASS',
#            '用例名称': '软件包文件检查',
#            '用例编号': 'common-base-01',
#            '错误日志': ''},
#           {'测试套件': 'Desktop',
#            '测试用时': '00:00:00',
#            '测试结果': 'PASS',
#            '用例名称': '包名规范一致性检查',
#            '用例编号': 'common-base-02',
#            '错误日志': ''},
#           {'测试套件': 'Desktop',
#            '测试用时': '00:00:01',
#            '测试结果': 'PASS',
#            '用例名称': '软件依赖库（so文件）查询测试',
#            '用例编号': 'common-compatibility-01',
#            '错误日志': ''},
#           {'测试套件': 'Desktop',
#            '测试用时': '00:00:00',
#            '测试结果': 'NA',
#            '用例名称': '软件依赖字段（Depends）兼容性检查测试',
#            '用例编号': 'common-compatibility-02',
#            '错误日志': '不涉及此项测试，不做测试'},
#           {'测试套件': 'Desktop',
#            '测试用时': '00:00:01',
#            '测试结果': 'PASS',
#            '用例名称': '软件gcc版本兼容性测试',
#            '用例编号': 'common-compatibility-03',
#            '错误日志': ''},
#           ]
#
# a = DocUtils(classify)
# result = a.output_document(env, result)
# print(result)
